<?php

require_once dirname(__DIR__, 6) . '/VtbPay/vendor/autoload.php';

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;

/**
 * A class intended for integration with the VTB payment system.
 * This handler allows you to set up and process payments via VTB in your online store.
 *
 * v 1.2.2
 *
 */
class VtbPayPayment extends payment
{
    /** @const string[] CURRENCIES поддерживаемые валюты */
    const CURRENCIES = ['RUB', 'RUR'];

    private $logger;


    /**
     * Checks whether the currency of the order is supported by the payment system
     *
     * @return bool Returns true if the order currency is supported, and false otherwise
     * @throws coreException Throws coreException when it encounters a system-level problem
     * @throws privateException Throws privateException if there's an issue with the private data
     */
    private function isSupportedCurrency() : bool
    {
        $currency = $this->getCurrencyCode();
        return in_array(strtoupper($currency), self::CURRENCIES);
    }


    /**
     * Checks if the order is suitable for processing by the payment system
     *
     * @param order $order The order to check
     * @return bool Returns true if the order is suitable, and false otherwise
     */
    public function isSuitable(order $order) : bool
    {
        return (
            $this->isSupportedCurrency() &&
            parent::isSuitable($order) &&
            ($order->getActualPrice() >= 0)
        );
    }


    /**
     * Retrieves the customer email from a customer object
     *
     * @param customer $customer The customer from which to retrieve the email
     * @return string The email of the customer
     * @throws publicException Throws publicException if the email is invalid
     */
    protected function getCustomerEmail(customer $customer)
    {
        $email = $customer->getEmail();

        if (
            !is_string($email) ||
            empty($email) ||
            !umiMail::checkEmail($email)
        ) {
            if (empty($email = $this->object->vtbpay_email_fiscal)) {
                throw new publicException(getLabel('error-payment-wrong-customer-email'));
            }
        }
        return $email;
    }


    /**
     * Retrieves the customer object based on the customer ID in the order
     *
     * @return customer Returns the customer object
     * @throws publicException Throws publicException if the customer ID is invalid
     */
    protected function getCustomer()
    {
        $customerId = $this->order->getCustomerId();
        $customerSource = selector::get('object')->id($customerId);

        if (!$customerSource instanceof iUmiObject) {
            throw new publicException(getLabel('error-payment-wrong-customer-id'));
        }

        return new customer($customerSource);
    }


    /**
     * Processes the order by creating a payment link and redirects to it
     *
     * @param string|null $template The template to use for processing
     * @throws Exception|publicException|privateException Throws exceptions when encountering errors during processing
     * @return null
     */
    public function process($template = null)
    {
        $order = $this->order;
        $orderId = $order->getId();
        $cmsController = cmsController::getInstance();
        $this->setVtbpayLogger();
        $this->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));

        try {
            // Order cost
            $amount = $this->formatPrice($order->getActualPrice());

            // Customer email
            $customer = $this->getCustomer();
            $customerEmail = $this->getCustomerEmail($customer);

            // Return Url
            $returnUrl = 'http://' . $cmsController->getCurrentDomain()->getHost() . '/emarket/gateway/' . $orderId;

            // Since in this CMS the order identifier does not change when the order composition changes,
            // therefore, to create a new identifier, we associate the identifier with the price.
            $orderIdAndAmount = $orderId . '-' . $amount;

            $enableFiscal = (bool) ($this->object->vtbpay_enable_fiscal ?: false);
            $items = [];
            if ($enableFiscal) $items = $this->getItems();

            // Логирование входных данных
            $this->logger->debug(
                __CLASS__ . ' > ' . __FUNCTION__ . ' > getOrderLink - INPUT: ', [
                'arguments' => [
                    'orderIdAndAmount' => $orderIdAndAmount,
                    'customerEmail' => $customerEmail,
                    'time' => time(),
                    'amount' => $amount,
                    'returnUrl' => $returnUrl,
                    'twoStage' => false,
                    'items' => $items
                ]
            ]);

            $response = $this->getVtbApi()->getOrderLink(
                $orderIdAndAmount,
                $customerEmail,
                time(),
                $amount,
                $returnUrl,
                false,
                $items
            );

            // Логирование результата
            $this->logger->debug(
                __CLASS__ . ' > ' . __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
                'response' => $response
            ]);

            if (
                !isset($response['object']['payUrl']) ||
                empty($payUrl = $response['object']['payUrl'])
            ) {
                throw new publicException('Failed to get payment URL.');
            }

            $order->setPaymentStatus('initialized', true);
            $order->order();

            $module = $cmsController->getModule('emarket');
            if ($module) {
                $module->redirect($payUrl);
            }

        } catch (\Exception | publicException | privateException | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            // Log the caught exception for debugging purposes.
            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: %s;',
                $e->getMessage(),
                $orderId ?: ''
            ), $context);

            throw $e;
        }
    }


    /**
     * Получает массив товаров заказа.
     *
     * @return array Массив товаров.
     */
    private function getItems(): array
    {
        $items = [];

        foreach ($this->order->getItems() as $key => $item) {
            $items[] = [
                'positionId' => ($key + 1),
                'name' => $item->getName(),
                'code' => (string) $item->getId(),
                'price' => floatval($item->getActualPrice()),
                'measure' => (int) $this->getValueSelectTypeObject('vtbpay-measure-fiscal') ?: 0,
                'quantity' => (int) $item->getAmount(),
                'taxParams' => [
                    'taxType' => $this->getValueSelectTypeObject('vtbpay-tax-type-fiscal') ?: 'none'
                ],
                'paymentType' => $this->getValueSelectTypeObject('vtbpay-payment-type-fiscal') ?: 'full_prepayment',
                'paymentSubject' => (int) $this->getValueSelectTypeObject('vtbpay-payment-subject-fiscal') ?: 1,
                'amount' => floatval($item->getActualPrice() * $item->getAmount())
            ];
        }

        $deliveryPrice = $this->order->getValue('delivery_price');
        if ($deliveryPrice > 0) {
            $delivery = selector::get('object')->id(
                $this->order->getValue('delivery_id')
            );

            if (!$delivery instanceof iUmiObject) {
                throw new expectObjectException(getLabel('error-unexpected-exception'));
            }
            $items[] = [
                'positionId' => ($key + 2),
                'name' => $delivery->getName() ?: 'Доставка',
                'price' => floatval($deliveryPrice),
                'measure' => 0,
                'quantity' => 1,
                'taxParams' => [
                    'taxType' => $this->getValueSelectTypeObject('vtbpay-tax-type-fiscal') ?: 'none'
                ],
                'paymentType' => $this->getValueSelectTypeObject('vtbpay-payment-type-delivery-fiscal') ?: 'full_prepayment',
                'paymentSubject' => 4,
                'amount' => floatval($deliveryPrice)
            ];
        }

        return $items;
    }


    /**
     * Checks the status of the payment and updates the payment status accordingly
     *
     * @throws Exception|publicException|privateException Throws exceptions when encountering errors during polling
     */
    public function poll()
    {
        $buffer = outputBuffer::current();
        $buffer->clear();
        $buffer->contentType('text/plain');
        $this->setVtbpayLogger();
        $this->logger->setOption('additionalCommonText', 'return-' . rand(1111, 9999));

        $orderIdAndAmount = getRequest('orderId');

        try {
            if (empty($orderIdAndAmount)) {
                throw new publicException('Error: Order ID not found.');
            }

            // Логирование входных данных
            $this->logger->debug(
                __CLASS__ . ' > ' . __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
                'arguments' => [
                    'orderIdAndAmount' => $orderIdAndAmount,
                ]
            ]);

            $response = $this->getVtbApi()->getOrderInfo($orderIdAndAmount);

            // Логирование результата
            $this->logger->debug(
                __CLASS__ . ' > ' . __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
                'response' => $response
            ]);

            if (
                isset($response['object']['status']['value']) &&
                $response['object']['status']['value'] == 'PAID'
            ) {
                $this->order->setPaymentStatus('accepted');
                $host = $this->getSuccessUrl();
            }
            else {
                $this->order->setPaymentStatus('failed');
                $host = $this->getFailUrl();

                // Logging unsuccessful payment
                $this->logger->error(sprintf(
                    __FUNCTION__ . ' > getOrderInfo. Payment not paid. Order ID and amount: %s',
                    $orderIdAndAmount
                ));
            }

            $cmsController = cmsController::getInstance();
            $module = $cmsController->getModule('emarket');
            if ($module) {
                $this->order->commit();
                $module->redirect($host);
            }
        } catch (\Exception | publicException | privateException | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            // Log the caught exception for debugging purposes.
            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order ID and amount: %s;',
                $e->getMessage(),
                $orderIdAndAmount ?: ''
            ), $context);

            throw $e;
        }
    }


    /**
     * Получает значение объекта типа выпадающий список
     *
     * @param string $typeName Название типа
     *
     * @return string Значение
     */
    private function getValueSelectTypeObject($typeName): string
    {
        $propertyName = str_replace('-', '_', $typeName);
        $object = umiObjectsCollection::getInstance()->getObject($this->object->$propertyName ?? NULL);
        if (empty($object)) return '';
        return str_replace($typeName . '-', '', $object->getGUID());
    }


    /**
     * Получает объект для работы с API ВТБ.
     *
     * @return VtbApi Объект для работы с API ВТБ.
     */
    public function getVtbApi(): VtbApi
    {
        return new VtbApi(
            $this->object->vtbpay_client_id,
            $this->object->vtbpay_client_secret,
            (bool) $this->object->vtbpay_test_mode,
            $this->object->vtbpay_merchant_authorization ?: ''
        );
    }


    /**
     * Устанавливает объект логирования для модуля.
     */
    public function setVtbpayLogger(): void
    {
        $logging = (bool) $this->object->vtbpay_logging;

        $sysLogPath = mainConfiguration::getInstance()->includeParam('sys-log-path');
        $logPath = sprintf('%s/payment/%s/', rtrim($sysLogPath, '/'), str_replace('Payment', '', get_class($this)));
        $fileName = 'vtbpay-' . date('d-m-Y');

        $logger = new \umiLogger($logPath);
        $logger->setFileName($fileName);
        $logger->separateByIp(false);

        $this->logger = VtbPayLogger::getInstance()
                        ->setLogFilePath($logPath . $fileName . '.log')
                        ->setCustomRecording(function($message) use ($logger)
                        {
                            // Логирование ошибки
                            $logger->push($message . PHP_EOL, false);
                            $logger->save();

                        }, VtbPayLogger::LOG_LEVEL_ERROR)
                        ->setCustomRecording(function($message) use ($logger, $logging)
                        {
                            // Логирование сообщения дебага
                            if ($logging) {
                                $logger->push($message . PHP_EOL, false);
                                $logger->save();
                            }

                        }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}
